local JUI = require("JUI.JUIlib")
local jutils = require("utility.jutils")

local console = {}

local callbacks = {}

local gui = JUI.JUIScene:new()

local bg = JUI.Frame:new()
bg:setPosition(JUI.Dimension:new(0.5, 0, 8, 8))
bg:setSize(JUI.Dimension:new(0.5, 1, -16, -16))
bg:setBackgroundColor(JUI.Color:new(0.1, 0.1, 0.1, 0.75))

JUI:parent(gui, bg)

local databox = JUI.Frame:new()
databox:setPosition(JUI.Dimension:new(0, 0, 0, 0))
databox:setSize(JUI.Dimension:new(1, 1, 0, -20))
databox:setBackgroundColor(JUI.Color:new(1,1,1,0))
JUI:parent(bg, databox)

local inputbox = JUI.InputForm:new()
inputbox:setPosition(JUI.Dimension:new(0, 1, 0, -20))
inputbox:setSize(JUI.Dimension:new(1, 0, 0, 20))
inputbox:setBackgroundColor(JUI.Color:new(0.1, 0.1, 0.1))
inputbox:setGrabFocusOnReturn(true)
inputbox:setClearOnReturn(true)
inputbox.textColor = JUI.Color:new(1,1,1)

inputbox.inputEvent:connect(function(text)
	local notrail = text:match("^%s*(.-)%s*$")

	console.addMessage(">"..notrail, "low")

	notrail = notrail:lower()

	local t = jutils.string.explode(notrail, " ")
	local command = t[1]
	local args = {}
	for i = 1, #t-1 do
		
		args[i] = t[i+1]

		if tonumber(args[i]) then args[i] = tonumber(args[i]) end
	end

	for _, func in pairs(callbacks) do
		func(command, args)
	end
end)

JUI:parent(bg, inputbox)

local consoleOpen = false

function console.init()

end

function console.onCommand(func)
	table.insert(callbacks, func)
end

function console.isActive()
	return consoleOpen
end

function console.keypressed(key)
	if key == "tab" then
		consoleOpen = not consoleOpen
		love.keyboard.setKeyRepeat(consoleOpen)
		if consoleOpen then
			inputbox:grabFocus()
		else
			inputbox:dropFocus(false)
		end
	end
	if consoleOpen then
		gui:keypressPass(key)
	end
end

function console.textinput(t)
	if consoleOpen then
		gui:textinputPass(t)
	end
end

local inc = 0

function console.moveUp(indices)
	for i, v in pairs(databox:getChildren()) do
		--print(i, v)
		v:setPosition(v.position+JUI.Dimension:new(0, 0, 0, -(indices*16)))
	end
end

function console.addMessage(message, type, size)
	
	local autofix = (size==nil)
	size = size or 1

	console.moveUp(size)

	local textobj = JUI.Label:new()
	textobj:setSize(JUI.Dimension:new(1, 0, 0, size*16))
	textobj:setBackgroundColor(JUI.Color:new(0, 0, 0, 0))
	textobj:setPosition(JUI.Dimension:new(0, 1, 0, -size*16))
	textobj:setBorderColor(JUI.Color:new(0, 0, 0, 0))
	textobj:setText(message)
	local color = {1,1,1}
	if type == "error" then
		color = {1, 0, 0}
	elseif type == "warning" then
		color = {1, 1, 0}
	elseif type == "good" then
		color = {0, 1, 0}
	elseif type == "low" then
		color = {0.7, 0.7, 0.7}
	end
	JUI:parent(databox, textobj)
	textobj.textColor = JUI.Color:new(color[1], color[2], color[3])

	local function fix()
		if textobj:isOverflowing() then
			console.moveUp(1)
			size = size + 1
			textobj:setSize(JUI.Dimension:new(1, 0, 0, 16*size))
		else
			return
		end
	end
	if autofix then
		fix()
	end

end

function console.update(dt)
	if consoleOpen then
		gui:update(dt)
	end
end

function console.draw()
	if consoleOpen then
		gui:render()
	end
end

return console