local physicalentity = require("entities.physicalentity")
local rendering = require("utility.rendering")
local jutils = require("utility.jutils")

local itemstack = physicalentity:subclass("Itemstack")

function itemstack:init(itemid, amount)
	physicalentity.init(self)
	
	self.boundingbox = jutils.point.new(4, 4)

	self.id = itemid
	self.amount = amount
	self.oscillation = 0
end

local function magnitude(x1, y1, x2, y2)
	return math.sqrt( (x2 - x1)^2 + (y2-y1)^2 )
end

function itemstack:update(dt)
	physicalentity.update(self, dt)
	self.oscillation = self.oscillation + 3*dt

	if self.dead then return end

	for idx, entity in pairs(self.world.entities) do
		
		if self ~= entity and entity:isA("Itemstack") and entity.dead == false then
			if self.position:distance(entity.position) <= 10 then
			--if magnitude(self.position.x, self.position.y, entity.position.x, entity.position.y) <= 10 then
				if self.id == entity.id then
					self.amount = self.amount + entity.amount
					entity.dead = true
				end
			end
		end

		if entity:isA("Player") then
			local dist = self.position:distance(entity.position) 
			if dist <= 30 then
				local attraction = 1.1
				self.velocity = self.velocity + (entity.position-self.position):unitvec() * ((30-dist)^attraction) --* (1/dist))

				if dist <= 10 then
					local amountleft = entity.inventory:addItem(self.id, self.amount)
					self.amount = amountleft
					if self.amount < 1 then
						self.dead = true
					end
				end
			end
		end
	end
end

function itemstack:draw(dt)
	--physicalentity.draw(self, dt)
	rendering:drawItem(self.id, self.position.x, self.position.y- (math.sin(self.oscillation))-1, 0.8, 0, 4, 4)
	if self.amount > 1 then
		rendering:drawItem(self.id, self.position.x-1, self.position.y-1 - (math.sin(self.oscillation))-1, 0.8, 0, 4, 4)
	end

	if self.amount > 10 then
		rendering:drawItem(self.id, self.position.x-2, self.position.y-2 - (math.sin(self.oscillation))-1, 0.8, 0, 4, 4)
	end

	if self.amount > 100 then
		rendering:drawItem(self.id, self.position.x-3, self.position.y-3 - (math.sin(self.oscillation))-1, 0.8, 0, 4, 4)
	end
end

return itemstack