local jutils = require("utility.jutils")
local config = require("config")
local entity = require("entities.entity")

local physicalentity = entity:subclass("PhysicalEntity")

function physicalentity:init()
	entity.init(self)
	

	self.velocity = jutils.point.new(0, 0)
	self.lastvalidposition = jutils.point.new(0, 0)
	self.mass = 1
	
	self.frozen = false
	self.falldist = 0
	self.falling = true
	self.fallthrough = false
	self.noclip = false
end

function physicalentity:updatePhysics(dt)

	-- apply friction
	if self.frozen == false then
		local force = (self.velocity.x * config.physics.FRICTION)
		self.velocity.x = self.velocity.x - force

		if math.abs(self.velocity.x) < (1/80) then self.velocity.x = 0 end

		-- clamp horizontal velocity to walkspeed

		-- apply gravity if falling
		if self.falling then
			self.velocity.y = self.velocity.y + (config.physics.GRAVITY*dt)
		end

		-- velocity clamping
		local TERMINAL = config.physics.TERMINAL_VELOCITY
		self.velocity.x = jutils.math.clamp(-TERMINAL, self.velocity.x, TERMINAL)
		self.velocity.y = jutils.math.clamp(-TERMINAL, self.velocity.y, TERMINAL)
	end

	self.position = self.nextposition
	self.nextposition = self.nextposition + (self.velocity * dt)
end

function physicalentity:update(dt)
	entity.update(self, dt)



	if self.frozen then
		--self.nextposition = self.position
		--return
	end

end

function physicalentity:draw()
	entity.draw(self)
end

return physicalentity