local config = require("config")
local worldclass = require("world.world")
local jutils = require("utility.jutils")
local grid = require("utility.grid")
local items = require("items")
local input = require("utility.input")
local rendering = require("utility.rendering")
local console = require("console")
local tiles = require("tiles")

local settingsmenu = require("settingsmenu")


local settingsOpen = false

local world = worldclass.new("world1")

local gameloop = {}

local camera = jutils.point.new(0, 0)
local scale = 2

local function makeplayer()
	return world:addEntity("player")
end

local playerEntity = makeplayer()
local playerdead = false
local respawntime = 0

console.onCommand(function(command, args)
	
	local function validArg(idx, typeNeeded)

		local arg = args[idx]

		if not arg then
			console.addMessage("argument #"..idx.." is missing", "error")
			return false
		end

		if type(arg) ~= typeNeeded then
			console.addMessage("argument #"..idx.." must be a "..typeNeeded, "error")
			return false
		end

		return true
	end

	if command == "help" then
		console.addMessage(
[[
List of console commands:
	help
	tp <x> <y>
	respawn
	exit
	give <item> <amount>
	itemlist, tilelist
	fast, fly, noclip
]], nil, 7)
		return
	end

	if command == "tp" then
		if validArg(1, "number") and validArg(2, "number") then
			playerEntity.nextposition = jutils.point.new(args[1], args[2])
			return
		end
		console.addMessage("command couldn't be ran!", "error")
		return
	end

	if command == "give" then
		local item = args[1]

		if validArg(1, "string") then

			local amount = 1
			if args[2] then
				if validArg(2, "number") then
					amount = args[2]
				else
					return
				end
			end

			local itemname = string.upper(args[1])
			if items[itemname] then
				local item = items[itemname]

				playerEntity.inventory:addItem(item.id, amount)
			else
				console.addMessage("item "..itemname.." doesn't exist!", "error")
			end

		end
		return
	end

	local function tits(val)
		if val == true then return "enabled" end
		if val == false then return "disabled" end
	end

	if command == "itemlist" then
		local num = items:getNumberOfItems()


		for index = 1, num-2, 3 do
			local tile1 = items:getByID(index)
			local tile2 = items:getByID(index+1)
			local tile3 = items:getByID(index+2)
			console.addMessage(tile1.name..", "..tile2.name..", "..tile3.name)
		end
		return
	end

	if command == "tilelist" then
		local num = tiles:getNumberOfTiles()


		for index = 1, num-3, 4 do
			local tile1 = tiles:getByID(index)
			local tile2 = tiles:getByID(index+1)
			local tile3 = tiles:getByID(index+2)
			local tile4 = tiles:getByID(index+3)
			console.addMessage(tile1.name..", "..tile2.name..", "..tile3.name..", "..tile4.name)
		end
		
		return
	end

	if command == "noclip" then
		playerEntity.noclip = not playerEntity.noclip

		local text = tits(playerEntity.noclip)

		console.addMessage("Noclip "..text)
		return
	end

	if command == "fly" then
		playerEntity.frozen = not playerEntity.frozen

		local text = tits(playerEntity.frozen)

		console.addMessage("Fly "..text)
		return
	end

	if command == "fast" then
		playerEntity.fast = not playerEntity.fast

		local text = tits(playerEntity.fast)

		console.addMessage("Fastmove "..text)
		return
	end


	if command == "respawn" then
		playerEntity.dead = true
		playerEntity = makeplayer()
		return
	end

	if command == "exit" then
		love.window.close()
		os.exit()
		return
	end

	console.addMessage("No such command: "..command, "error")
end)


function gameloop.initialize()
	math.randomseed(config.generator.SEED)

	console.addMessage("cave game started...", "good")
end

function gameloop.clear()

end

function gameloop.wheelmoved(x, y)
	input.wheelmoved:call(x, y)
	--playerEntity:scroll(y)
end

function gameloop.keypressed(key)
	input.keypressed:call(key)
	console.keypressed(key)
end

function gameloop.keyreleased(key)
	input.keyreleased:call(key)
end

function gameloop.mousepressed(x, y, button, istouch, presses)
	input.mousepressed:call(x, y, button, istouch, presses)
end

function gameloop.mousereleased(x, y, button, istouch, presses)
	input.mousereleased:call(x, y, button, istouch, presses)
end

function gameloop.textinput(t)
	input.textinput:call(t)
	console.textinput(t)
end

function gameloop.update(dt)
	if settingsOpen then
		settingsmenu:update(dt)
		return
	end
	
	console.update(dt)
	if not console.isActive() then
		rendering:update(dt)

		camera = camera:lerp(playerEntity.position, 0.1)
		local px, py = grid.pixelToTileXY(camera.x, camera.y)
		local camx, camy = grid.tileToChunkXY(px, py)
		local fp = jutils.point.new(camx, camy)

		world:setFocusPoint(fp)

		if playerEntity.dead == true then playerdead = true end

		if playerdead then
			respawntime = respawntime + dt

			if respawntime > 3 then
				playerdead = false
				respawntime = 0
				playerEntity = makeplayer()
			end
		end

		

		world:update(dt)

	end

end

function gameloop.draw()
	local ww, wh = love.graphics.getWidth(), love.graphics.getHeight()
	local ratioh = math.max(wh / 600, 0)
	local ratiow = math.max(ww / 1000, 0)
	local largerratio = math.max(ratioh, ratiow)

	love.graphics.push()
	love.graphics.translate(love.graphics.getWidth() / 2, love.graphics.getHeight() / 2)
	love.graphics.scale(scale * largerratio, scale * largerratio)
	love.graphics.translate(-camera.x, -camera.y)

	local mx, my = love.graphics.inverseTransformPoint(love.mouse.getX(), love.mouse.getY())

	input.setTransformedMouse(math.floor(mx), math.floor(my))

	mx, my = input.getTransformedMouse()
	-- gameworld rendering
	do
		world:draw()

		if playerdead then
			love.graphics.push()
			love.graphics.origin()
			love.graphics.setColor(0.9, 0, 0)
			rendering:setFont("pixelade_large")
			love.graphics.printf(
				"it died, let me make another real quick",
				0,
				love.graphics.getHeight() / 2.25,
				love.graphics.getWidth(),
				"center"
			)
			love.graphics.setFont("default")
			love.graphics.pop()
		end
		
	end

	love.graphics.pop()
	love.graphics.setColor(1, 1, 1)

	console.draw()

	if settingsOpen then
		settingsmenu:render()
	end
end

return gameloop