local jutils = require("utility.jutils")
local items = require("items")
local rendering = require("utility.rendering")
local object = require("utility.object")
local input = require("utility.input")

local inventory = object:subclass("Inventory")

function inventory:init(size, pos, slotsize)
	self.open = false
	self.size = size
	self.screenpos = pos
	self.slotsize = slotsize
	self.paddingsize = 3
	self.pointingat = -1
	self.items = {}
	self.itemDragging = {0, 0}

	-- initalize items field

	for index = 1, self.size.x * self.size.y do

		self.items[index] = {0, 0}
	end

	self.uiresponsive = false
end

function inventory:clicked(x, y, button, touch, number)
	local inventoryHandled = false
	if self.uiresponsive then
		
		-- clicked on an inventory spot
		if self.pointingat ~= -1 then
			

			local slot = self.items[self.pointingat]
			local sid, samount = slot[1], slot[2]

			local mid, mamount = self.itemDragging[1], self.itemDragging[2]

			if button == 1 then
				if mid == sid then
					slot[2] = slot[2] + mamount
					self.itemDragging = {0, 0}
				else
					self.itemDragging = {sid, samount}
					self.items[self.pointingat] = {mid, mamount}
				end
				inventoryHandled = true
			elseif button == 2 then
				-- item is being dragged, put 1 back in inventory
				--[[if mid > 0 then
					if mid == sid or sid == 0 then
						self.itemDragging[2] = self.itemDragging[2] - 1
						slot[1] = mid
						slot[2] = slot[2] + 1
						inventoryHandled = true
					end
				else]]
				if sid > 0 then
					if mid == sid or mid == 0 then
						
						self.itemDragging[1] = sid
						self.itemDragging[2] = self.itemDragging[2] + 1
						slot[2] = slot[2] - 1
						inventoryHandled = true
					end
				end

			end
		end
	elseif self.itemDragging[1] > 0 then
		if button == 2 then
			-- TODO: make itemstack drop
		end
	end
	return inventoryHandled
end

function inventory:getSlotIndex(x, y)
	local index = ((y - 1) * self.size.x) + x
	return index
end

function inventory:getSlotXY(index)
	local x = (index - 1) % (self.size.x) + 1
	local y = math.floor((index - 1) / self.size.x) + 1

	return x, y
end

function inventory:getSlot(index)
	return self.items[index][1], self.items[index][2]
end

function inventory:setSlot(index, itemid, amount)
	self.items[index] = {itemid, amount}
end

function inventory:addItem(item, amount)
	local amountleft = amount
	for index, data in pairs(self.items) do
		local itemdata = items:getByID(item)

		-- itemstack is the same or is empty
		if amountleft < 1 then return 0 end
		if data[1] == item or data[1] == 0 then
			data[1] = item

			-- stack isn't full
			while (itemdata.stack > data[2]) and amountleft > 0 do
				data[2] = data[2] + 1
				amountleft = amountleft - 1
			end
		end
	end
	return amountleft
end

function inventory:hasItem(item, amount)
	local amounthas = 0

	for index, data in pairs(self.items) do
		-- itemstack is the same or is empty
		if data[1] == item then
			amounthas = amounthas + data[2]
		end
	end
	

	if amounthas >= amount then
		return true, amounthas
	end
	return false
end

-- TODO: this still doesn't work right
function inventory:removeItem(item, amount)

end

function inventory:getDrawPosition()
	local x = (self.screenpos[1] * love.graphics.getWidth()) + self.screenpos[3]
	local y = (self.screenpos[2] * love.graphics.getHeight()) + self.screenpos[4]

	return x, y
end

function inventory:input()
	local screenx, screeny = self:getDrawPosition()


	local mx = love.mouse.getX()
	local my = love.mouse.getY()
	self.pointingat = -1
	for index = 1, #self.items do

		local x, y = self:getSlotXY(index)
		local drawx = screenx + (x - 1) * self.slotsize + (x * self.paddingsize)
		local drawy = screeny + (y - 1) * self.slotsize + (y * self.paddingsize)

		
		if mx > drawx and mx < drawx+self.slotsize then
			if my > drawy and my < drawy+self.slotsize then
				self.pointingat = index
			end
		end	
	end
end

function inventory:update(dt)
	for idx, stack in pairs(self.items) do
		if stack[2] < 1 then stack[1] = 0 end
		if stack[1] == 0 then stack[2] = 0 end
	end

	if self.itemDragging[2] < 1 then self.itemDragging[1] = 0 end
	if self.itemDragging[1] == 0 then self.itemDragging[2] = 0 end
end

rendering:setFont("pixelade_normal")
local text = love.graphics.newText(love.graphics.getFont())
rendering:setFont("default")
local lastpointed = nil

local function newText()
	text:clear()

end

local ITEM_DEBUG_INFO = true

local rarityColors = {
	[-1] = {0, 0, 0},
	[0] = {0.6, 0.6, 0.6},
	[1] = {1,1,1},
	[2] = {0.5, 1, 0.5},
	[3] = {0.5, 0.5, 1},
	[4] = {1, 0.5, 0.5},
	[5] = {1, 0.5, 1},
	[6] = {1, 1, 0.5},
}

function inventory:drawpointingdata()
	local dx, dy = self:getDrawPosition()
	local mx, my = dx+(self.size.x*(self.slotsize+self.paddingsize))+5, dy
	if self.pointingat ~= -1 then

		local item = self.items[self.pointingat]
		
		if item[1] > 0 then
			local iteminfo = items:getByID(item[1])

			if item ~= lastpointed then
				newText()
			end
			lastpointed = item
			local f = love.graphics.getFont()
			local color = rarityColors[iteminfo.rarity]
			text:add({color, iteminfo.displayname}, 0, 0, 0, 1.1, 1.1)
			
			if iteminfo.tooltip then
				local function substitute(input, valuename, func)
					local sub = ""
					
						
					if iteminfo[valuename] then
						sub = iteminfo[valuename]
						if func then
							sub = func(iteminfo[valuename])
						end

						input = input:gsub("{"..valuename.."}", sub)
					end
					return input
				end

				local tooltip = iteminfo.tooltip

				tooltip = substitute(tooltip, "usedistance")
				tooltip = substitute(tooltip, "strength")
				tooltip = substitute(tooltip, "speed", function(n)return 1/n end)

				text:add({{0.7,0.7,0.7},tooltip}, 0, f:getHeight()+8)
			end

			if ITEM_DEBUG_INFO then

				text:add({})
			end
			
			love.graphics.draw(text, mx, my)

		end
	end
end

function inventory:drawSlotHighlighted(index, color)
	local screenx, screeny = self:getDrawPosition()

	local x, y = self:getSlotXY(index)
	local drawx = (x - 1) * self.slotsize + (x * self.paddingsize)
	local drawy = (y - 1) * self.slotsize + (y * self.paddingsize)

	local growth = 0
	local rx, ry = 0, 0

	love.graphics.setColor(color)
	growth = 2
	rx = 4
	ry = 4

	love.graphics.rectangle("fill", screenx + drawx - growth, screeny + drawy - growth, self.slotsize + (growth*2), self.slotsize + (growth*2), rx, ry)

	local itemdata = self.items[index]

	if itemdata[1] > 0 then
		local iteminfo = items:getByID(itemdata[1])
		rendering:drawItem(itemdata[1], screenx+drawx+2 - (growth/2), screeny+drawy+2 - (growth/2), (self.slotsize+growth)/9, 0)

		if itemdata[2] > 1 then
			love.graphics.setColor(0,0,0)
			rendering:setFont("pixelade_normal")
			love.graphics.printf(itemdata[2], screenx+drawx, screeny+drawy+15, 32, "right")
			rendering:setFont("default")
		end
	end
end

function inventory:drawSlot(index)

	local screenx, screeny = self:getDrawPosition()

	local x, y = self:getSlotXY(index)
	local drawx = (x - 1) * self.slotsize + (x * self.paddingsize)
	local drawy = (y - 1) * self.slotsize + (y * self.paddingsize)

	local growth = 0
	local rx, ry = 0, 0

	love.graphics.setColor(1, 1, 1, 0.5)
	love.graphics.rectangle("fill", screenx + drawx - growth, screeny + drawy - growth, self.slotsize + (growth*2), self.slotsize + (growth*2), rx, ry)

	local itemdata = self.items[index]

	if itemdata[1] > 0 then
		local iteminfo = items:getByID(itemdata[1])
		rendering:drawItem(itemdata[1], screenx+drawx+2 - (growth/2), screeny+drawy+2 - (growth/2), (self.slotsize+growth)/9, 0)

		if itemdata[2] > 1 then
			love.graphics.setColor(0,0,0)
			rendering:setFont("pixelade_normal")
			love.graphics.printf(itemdata[2], screenx+drawx, screeny+drawy+15, 32, "right")
			rendering:setFont("default")
		end
	end
end

function inventory:drawDragging()
	local x, y = love.mouse.getPosition()
	local growth = 0
	local rx, ry = 0, 0
	local itemdata = self.itemDragging

	if itemdata[1] > 0 then
		local iteminfo = items:getByID(itemdata[1])
		rendering:drawItem(itemdata[1], x - (growth/2), y - (growth/2), (self.slotsize+growth)/9, 0)

		if itemdata[2] > 1 then
			love.graphics.setColor(0,0,0)
			rendering:setFont("pixelade_normal")
			love.graphics.printf(itemdata[2], x, y+15, 32, "right")
			rendering:setFont("default")
		end

	end
end

function inventory:draw()
	if not self.open then return end

	love.graphics.push()
	love.graphics.origin()
	local screenx, screeny = self:getDrawPosition()

	for index = 1, #self.items do
		if index == self.pointingat then
			self:drawSlotHighlighted(index, {1,1,1})
		else
			self:drawSlot(index)
		end
	end
	self:drawDragging()
	self:drawpointingdata()
	love.graphics.pop()
end

return inventory
