local jutils = require("utility.jutils")
local items = require("items")
local rendering = require("utility.rendering")

local inventory = require("inventory.inventory")

local playerinventory = inventory:subclass("PlayerInventory")

local openPos = {0, 1, 2, -180}
local closedPos = {0, 1, 2, -40}

function playerinventory:init()
	inventory.init(self, jutils.point.new(10, 4), closedPos, 32)
	self.fullyshown = false
	self.hotbarslot = 1
end

function playerinventory:fullOpen()
	self.screenpos = openPos
	self.uiresponsive = true
	self.fullyshown = true
	self.pointingat = -1
end

function playerinventory:fullClose()
	self.screenpos = closedPos
	self.uiresponsive = false
	self.fullyshown = false
end

function playerinventory:getHeldItem()

	if self.itemDragging[1] > 0 then
		return self.itemDragging
	end
	return self.items[self.hotbarslot]
end

function playerinventory:update(dt)
	inventory.update(self, dt)

	if self.fullyshown then
		self:input()
	else
		--self.pointingat = self.hotbarslot
	end
end

local rarityColors = {
	[-1] = {0, 0, 0},
	[0] = {0.6, 0.6, 0.6},
	[1] = {1,1,1},
	[2] = {0.5, 1, 0.5},
	[3] = {0.5, 0.5, 1},
	[4] = {1, 0.5, 0.5},
	[5] = {1, 0.5, 1},
	[6] = {1, 1, 0.5},
}

function playerinventory:drawhotbar()
	local screenx, screeny = self:getDrawPosition()

	local item = self.items[self.hotbarslot]
	if item[1] > 0 then
		local iteminfo = items:getByID(item[1])
		love.graphics.setColor(rarityColors[iteminfo.rarity])
		rendering:setFont("pixelade_normal")
		love.graphics.print(iteminfo.displayname, screenx+2, screeny-20)
		rendering:setFont("default")
	end
end

function playerinventory:draw()
	love.graphics.push()
	love.graphics.origin()
	if self.fullyshown then

		love.graphics.setColor(0, 0, 0, 0.3)
		love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight())

		for index = 1, #self.items do
			if index == self.pointingat then
				self:drawSlotHighlighted(index, {1,1,1})
			elseif index == self.hotbarslot then
				self:drawSlotHighlighted(index, {1,1,1, 0.25})
			else
				self:drawSlot(index)
			end
		end
		self:drawpointingdata()
	else
		for index = 1, 10 do
			if index == self.hotbarslot then
				self:drawSlotHighlighted(index, {1,1,1})
			else
				self:drawSlot(index)
			end
		end
		self:drawhotbar()
	end

	self:drawDragging()
	love.graphics.pop()
end

return playerinventory