local config = require("config")
local jutils = require("utility.jutils")
local gameloop = require("gameloop")


function love.load()

	love.window.setTitle("Cave Game 8-22-19")
	love.window.setMode(
		1000,
		600,
		{
			fullscreen = false,
			resizable = true,
			minwidth = 500,
			minheight = 300,
			vsync = false
		}
	)

	gameloop.initialize()
	
end

function love.conf(t)
	t.version = "11.2"
end

function love.keypressed(key)
	gameloop.keypressed(key)
end

function love.keyreleased(key)
	gameloop.keyreleased(key)
end

function love.mousepressed(...)
	gameloop.mousepressed(...)
end

function love.mousereleased(...)
	gameloop.mousereleased(...)
end

function love.wheelmoved(x, y)
	gameloop.wheelmoved(x, y)
end

function love.textinput(t)
	gameloop.textinput(t)
end


function love.update(dt)

	--require("lurker").update()
	gameloop.update(dt)

end

local statvalue = {}
local statlabel = {}
local statssuffix = {}

local idx = 1

local function addStat(label, value, suffix)
	statvalue[idx] = value
	statlabel[idx] = label
	statssuffix[idx] = (suffix ~= nil) and suffix or ""
	idx = idx + 1
end

local function drawstats()
	local text = ""

	for index, label in ipairs(statlabel) do
		local value = statvalue[index]
		local suffix = statssuffix[index]

		local addition = ""
		if type(value) == "table" then
			for idx, inst in ipairs(value) do
				if idx < #value then
					addition = addition .. inst .. ","
				else
					addition = addition .. inst
				end
			end
		else
			addition = tostring(value)
		end
		text = text .. label .. ": " .. addition .. suffix .. " "
	end

	love.graphics.setColor(1, 1, 1)
	love.graphics.print(text, 2, 2)
end

function love.draw()
	
	gameloop.draw()
	
	local stats = love.graphics.getStats()
	local luavmmem = collectgarbage("count")

	if config.debug.SHOW_DATA then
		addStat("fps", love.timer.getFPS())
		addStat("dt", jutils.math.round(love.timer.getAverageDelta() * 1000, 3), "ms")
		addStat("luavm-memory", jutils.math.round(luavmmem / 1000, 1), "mb")

		addStat("\nimg", stats.images)
		addStat("txmem", jutils.math.round(stats.texturememory / 1000, 1), "kb")
		addStat("dc", stats.drawcalls)
		addStat("batched-dc", stats.drawcallsbatched)

		drawstats()

		statssuffix = {}
		statlabel = {}
		statvalue = {}
		idx = 1
	else
		love.graphics.setColor(1, 1, 1)
		love.graphics.print("fps: " .. love.timer.getFPS(), 2, 2)
	end

	love.graphics.setColor(0, 0, 0, 0.5)

end
