local tiles = require("tiles")
local items = require("items")
local config = require("config")
local renderer = {}

local frametimer = 0
local defaultQuad = love.graphics.newQuad(0, 0, 8, 8, 8, 8)

local fonts = {
	pixelade_tiny = love.graphics.newFont("assets/fonts/pixelade.ttf", 10.5),
	pixelade_small = love.graphics.newFont("assets/fonts/pixelade.ttf", 11),
	pixelade_normal = love.graphics.newFont("assets/fonts/pixelade.ttf", 22),
	pixelade_large = love.graphics.newFont("assets/fonts/pixelade.ttf", 40),

	arcade_tiny = love.graphics.newFont("assets/fonts/ffforwa.ttf", 9),
	arcade_small = love.graphics.newFont("assets/fonts/ffforwa.ttf", 10),
	arcade_normal = love.graphics.newFont("assets/fonts/ffforwa.ttf", 14),
	arcade_large = love.graphics.newFont("assets/fonts/ffforwa.ttf", 30),

	default = love.graphics.newFont(12)
}

function renderer:update(dt)
	frametimer = frametimer + dt
end

local damageimage = love.graphics.newImage("assets/damage.png")

function renderer:draw(tileid, state, dmg, light, tx, ty)
	local data = tiles:getByID(tileid)

	if data.customrender then
		data.customrender(state, dmg, light, tx, ty)
	else

		local damage = (data.hardness-dmg)/data.hardness

		local quad = defaultQuad


		if data.quad then quad = data.quad end

		if data.animation then
			local animFrame = math.floor(frametimer * data.animationspeed)

			local animNum = (animFrame % (#data.animation)) + 1
			quad = data.animation[animNum]
		end



		local r = data.color[1] * light
		local g = data.color[2] * light
		local b = data.color[3] * light 

		love.graphics.setColor(r, g, b)
		love.graphics.draw(
			data.texture,
			quad,
			(tx) * config.TILE_SIZE,
			(ty) * config.TILE_SIZE,
			0,
			config.TILE_SIZE / 8,
			config.TILE_SIZE / 8
		)
		if dmg > 0 then
			love.graphics.setColor(1,1,1, 1-damage)
			love.graphics.draw(damageimage, 
			(tx) * config.TILE_SIZE,
			(ty) * config.TILE_SIZE,
			0,
			config.TILE_SIZE / 8,
			config.TILE_SIZE / 8)
		end
	end
end

function renderer:drawItem(itemid, x, y, scaling, rotation, offx, offy)
	local data = items:getByID(itemid)

	local quad = defaultQuad
	if data.animation then
		local animFrame = math.floor(frametimer * data.animationspeed)

		local animNum = (animFrame % (#data.animation)) + 1
		quad = data.animation[animNum]
	end

	love.graphics.setColor(data.color)
	love.graphics.draw(
		data.texture,
		quad,
		x,
		y,
		rotation,
		scaling,
		scaling,
		offx or 0,
		offy or 0
	)
end

function renderer:setFont(fontname)
	assert(fonts[fontname] ~= nil, "Font doesn't exist!")
	love.graphics.setFont(fonts[fontname])
end

return renderer
