--[[
	World generation script.
]]

local generator = {}


local config = require("config")
local tiles = require("tiles")

local basefreq = 15
local sys = 30

local jutils = require("utility.jutils")
local noise = require("utility.noise")

local function skyIslandPass(chunk)
	local worldx = chunk.position.x*config.CHUNK_SIZE
	local worldy = chunk.position.y*config.CHUNK_SIZE
	for x = 1, config.CHUNK_SIZE do
		for y = 1, config.CHUNK_SIZE do

			local tile = tiles.AIR.id
			local level = noise.surfacenoise(worldx+x, worldy+y)

			if y+worldy < (config.generator.SURFACE_HEIGHT-1000) then

				local islandnoise = noise.noise(worldx+x, worldy+y, 2000, 500)

				if islandnoise > 0.4 then
					chunk.tiles[x][y] = tiles.STONE.id
				end
			end
		end
	end
end


local function chunkSurfacePass(chunk)
	local worldx = chunk.position.x*config.CHUNK_SIZE
	local worldy = chunk.position.y*config.CHUNK_SIZE
	for x = 1, config.CHUNK_SIZE do
		for y = 1, config.CHUNK_SIZE do

			local tile = tiles.AIR.id
			local level = noise.surfacenoise(worldx+x, worldy+y)

			--[[if level > 0 then
				if chunk.tiles[x][y] == tiles.AIR then
					chunk.tiles[x][y] = tiles.STONE
				end
			end]]

			

			if level > 0 then
				if chunk.tiles[x][y] == tiles.AIR.id then

					local stonecolor = noise.noise(worldx+x, worldy+y, 512, 512)

					if stonecolor > 0.25 then
						chunk.tiles[x][y] = tiles.STONE_RED.id
					elseif stonecolor > 0 then
						chunk.tiles[x][y] = tiles.STONE_YELLOW.id
					elseif stonecolor > -0.25 then
						chunk.tiles[x][y] = tiles.STONE_BROWN.id
					else
						chunk.tiles[x][y] = tiles.STONE.id
					end


					-- will be grass above..
					if noise.surfacenoise(worldx+x, worldy+y-1) <= 0 then
						chunk.tiles[x][y] = tiles.GRASS.id
						--[[for i = 1, 8 do
							--if y+i <= mapsize then
							chunk.tiles[x][y+i] = tiles.DIRT
						end]]
					elseif noise.surfacenoise(worldx+x, worldy+y-20) <= 0 then

						chunk.tiles[x][y] = tiles.DIRT.id
					elseif noise.surfacenoise(worldx+x, worldy+y-30) <= 0 then
						local chance = -noise.surfacenoise(worldx+x, worldy+y-30)

						local rand = math.random(1, 20)

						if chance > rand then
							chunk.tiles[x][y] = tiles.DIRT.id
						end
					end
				end
			end
		end
	end
end

local function chunkCavernPass(chunk)
	local worldx = chunk.position.x*config.CHUNK_SIZE
	local worldy = chunk.position.y*config.CHUNK_SIZE

	for x = 1, config.CHUNK_SIZE do
		for y = 1, config.CHUNK_SIZE do


			local dirtpile = noise.noise(worldx+x, worldy+y, 128, 128)

			if dirtpile > 0.35 and
				   (chunk.tiles[x][y] == tiles.STONE.id or
					chunk.tiles[x][y] == tiles.STONE_BROWN.id or
					chunk.tiles[x][y] == tiles.STONE_RED.id or
					chunk.tiles[x][y] == tiles.STONE_YELLOW.id) then
				chunk.tiles[x][y] = tiles.DIRT.id
			end


			local surface = noise.surfacenoise(worldx+x, worldy+y)


			-- vertical caves
			local function normalcave()
				local cavepossible = noise.noise(worldx+x, worldy+y, 512, 512)
				local cavenoise = 0.5 * noise.noise(worldx+x, worldy+y, 128, 128)
				local cavenoise2 = 0.25 * noise.noise(worldx+x, worldy+y, 32, 48)

				local docave = cavepossible * cavenoise * cavenoise2

				if surface < 20 then

					-- deep cave generation
					
					if cavepossible > 0 and cavepossible < 0.3 and docave > 0.15 then
						chunk.tiles[x][y] = tiles.AIR.id
					end
				else
					-- shallow cave generation

					if cavepossible > 0 and cavepossible < 0.3 and docave > 0 then
						chunk.tiles[x][y] = tiles.AIR.id
					end
				end
			end

			local function deepcave()
				local cavepossible = noise.noise(worldx+x, (worldy+y), 400, 1200)
				local cavenoise = noise.noise(worldx+x, worldy+y, 64, 64)
				if surface > 200 then
					-- shallow cave generation

					if cavepossible > 0.45 and cavenoise > -0.2 then
						chunk.tiles[x][y] = tiles.AIR.id
					end
				end
			end

			local function ravine()
				local cavepossible = noise.noise(worldx+x, (worldy+y), 250, 3000)
				if surface < 300 and surface > -10 then
					-- shallow cave generation

					if cavepossible > 0.49 then
						chunk.tiles[x][y] = tiles.AIR.id
					end
				end
			end

			normalcave()
			deepcave()
			--ravine()


		end

	end

end



local function chunkOrePass(chunk)
	local worldx = chunk.position.x*config.CHUNK_SIZE
	local worldy = chunk.position.y*config.CHUNK_SIZE

	--[[
	noisescale - affects the size of ore chunks
	threshold - affects the rarity of ore chunks
]]
	local function oretest(tile, x, y, offset, noisescale, threshold)
		local noise = noise.noise(worldx+x+offset, worldy+y+offset, noisescale, noisescale)

		if noise > threshold then
			chunk.tiles[x][y] = tile
		end

	end


	for x = 1, config.CHUNK_SIZE do
		for y = 1, config.CHUNK_SIZE do


			local bluegrassBiome = noise.noise(worldx+x, worldy+y, 1024, 1024)

			local bluegrassTerrain = noise.noise(worldx+x, worldy+y, 20, 20)

			bluegrassTerrain = jutils.math.round(bluegrassTerrain, 1)

			if worldy+y > 1000 and bluegrassBiome > 0.42 then


				if bluegrassTerrain > 0 then
					chunk.tiles[x][y] = tiles.MUD.id
				elseif bluegrassTerrain == 0 then
					chunk.tiles[x][y] = tiles.BLUE_GRASS.id
				elseif bluegrassTerrain < 0 then
					chunk.tiles[x][y] = tiles.AIR.id
				end

			end

			if chunk.tiles[x][y] ~= tiles.AIR.id then
				
				-- iron, copper, and tin generation
				oretest(tiles.IRON_ORE.id, x, y, 420, 28, 0.487)

				oretest(tiles.COPPER_ORE.id, x, y, 69, 48, 0.485)

				oretest(tiles.TIN_ORE.id, x, y, -666, 32, 0.482)

				-- gold, lead, aluminium

				if worldy+y > 400 then
					oretest(tiles.GOLD_ORE.id, x, y, 123, 20, 0.49)
					oretest(tiles.LEAD_ORE.id, x, y, -144, 24, 0.492)
					oretest(tiles.ALUMINIUM_ORE.id, x, y, 357, 64, 0.49)
				end

				-- cobalt, titanium
				if worldy+y > 1000 then
					oretest(tiles.COBALT_ORE.id, x, y, -404, 40, 0.495)
					oretest(tiles.TITANIUM_ORE.id, x, y, -222, 48, 0.49)
				end
				-- titanium, uranium, cobalt and 
				if worldy+y > 2400 then
					oretest(tiles.URANIUM_ORE.id, x, y, 237, 24, 0.495)
				end
			end
		end
	end
end

return function(chunk)
	chunkSurfacePass(chunk)
	chunkOrePass(chunk)
	chunkCavernPass(chunk)
	
	--skyIslandPass(chunk)
end